unit ComPort;
//http://alexmogurenko.com/blog/programming/windows-working-asynchronously-with-com-port-on-delphilazarus/
interface

uses
  Classes, Windows, SysUtils;

type
  TComPort = class(TObject)
    FBaudRate : Integer;
    FName : string;
    FHandle : THandle;
    m_pBuf : Pointer;
    FFlags : DWORD;
    FReadTimeOut : DWORD;
    FWriteTimeOut : DWORD;
  private
    procedure SetReadTimeOut(Value: DWORD);
    procedure SetWriteTimeOut(Value: DWORD);
  public
    Constructor Create;
    Destructor Destroy; override;
    //function Open(Baud : Integer; Index : Integer) : Boolean;

    function Connect(PortName : String; Baud : Integer) : Boolean;

    function InstanceActive () : Boolean;
    procedure CloseSocket();
    function Read() : String;
    function Write(AData : string) : boolean;
    property ReadTimeOut : DWORD read FReadTimeOut write SetReadTimeOut;
    property WriteTimeOut : DWORD read FWriteTimeOut write SetWriteTimeOut;
    function WaitingData: cardinal;
    procedure SendByte(data: byte);
    function RecvByte(ignored: integer) : byte;
    procedure Purge; //WARNING: DOES NOTHING : PROVIDED FOR SYNASER COMPATIBILITY
    procedure Flush; //WARNING: DOES NOTHING : PROVIDED FOR SYNASER COMPATIBILITY
    procedure SetBreak(Duration: integer); //WARNING: DOES NOTHING : PROVIDED FOR SYNASER COMPATIBILITY
    //property Binary : Boolean read GetBinary write SetBinary;
  end;

implementation
{Flags http://www.delphipages.com/forum/showthread.php?t=36366
dcb_Binary = $00000001;
dcb_ParityCheck = $00000002;
dcb_OutxCtsFlow = $00000004;
dcb_OutxDsrFlow = $00000008;
dcb_DtrControlMask = $00000030;
dcb_DtrControlDisable = $00000000;
dcb_DtrControlEnable = $00000010;
dcb_DtrControlHandshake = $00000020;
dcb_DsrSensivity = $00000040;
dcb_TXContinueOnXoff = $00000080;
dcb_OutX = $00000100;
dcb_InX = $00000200;
dcb_ErrorChar = $00000400;
dcb_NullStrip = $00000800;
dcb_RtsControlMask = $00003000;
dcb_RtsControlDisable = $00000000;
dcb_RtsControlEnable = $00001000;
dcb_RtsControlHandshake = $00002000;
dcb_RtsControlToggle = $00003000;
dcb_AbortOnError = $00004000;
dcb_Reserveds = $FFFF8000;


}

Constructor TComPort.Create;
//Constructor TComPort.Create(Baud : Integer; Index : Integer);
begin
  FFlags := 145;     //dcb_Binary dcb_DtrControlEnable dcb_TXContinueOnXoff
  FReadTimeOut := 300;
  FWriteTimeOut := 300;
  FHandle := INVALID_HANDLE_VALUE;
  m_pBuf := GetMemory(1024);
  inherited Create;
end;

Destructor TComPort.Destroy;
begin
  FreeMemory(m_pBuf);
  Inherited Destroy;
end;

procedure TComPort.Purge;
begin
    //
end;

procedure TComPort.Flush;
begin
    //
end;

procedure TComPort.SetBreak(Duration: integer);
begin
   //
end;

procedure TComPort.SetReadTimeOut(Value: DWORD);
begin
  FReadTimeOut := Value;
end;

procedure TComPort.SetWriteTimeOut(Value: DWORD);
begin
  FWriteTimeOut := Value;
end;

{
function TComPort.GetBinary() : Boolean;
begin
  Result := (FFlags and $00000001) = $00000001;
end;

procedure TComPort.SetBinary(Value : Boolean);
begin
  if (Value) then
  begin
    FFlags := FFlags or ($00000001 shl 0);
  end else
  begin
    FFlags := FFlags and (not($00000001 shl 0));
  end;
end;
}

function TComPort.InstanceActive () : Boolean;
begin
  if  (FHandle = INVALID_HANDLE_VALUE) then
    result := false
  else
    result := true;
end;

function TComPort.WaitingData: cardinal;
var
  Errors: Cardinal;
  CommStat: TComStat;
begin
  if not ClearCommError(FHandle, Errors, @CommStat) then
  begin
    PurgeComm(FHandle, PURGE_RXCLEAR);     //Just empty comm buffer on error and return 0
    result := 0;
  end else
    result := CommStat.cbInQue;
end;

//function TComPort.Open(Baud : Integer; Index : Integer) : Boolean;
function TComPort.Connect(PortName : String; Baud : Integer) : Boolean;  //'COM5',115200
var
  PortDCB : TDCB;
  Timeouts : COMMTIMEOUTS;
begin
  Result := False;
  FBaudRate := Baud;
  FName := PortName; //'COM' + IntToStr(Index);
  FHandle := CreateFile(PChar(FName),
                        GENERIC_READ or GENERIC_WRITE,
                        0, nil, OPEN_EXISTING,
			0, 0);
  if (FHandle = INVALID_HANDLE_VALUE) then
    exit;
  FillChar(PortDCB, sizeof(TDCB), #0);
  PortDCB.DCBlength := sizeof(TDCB);
  GetCommState(FHandle, PortDCB);
  PortDCB.BaudRate := FBaudRate;
  PortDCB.ByteSize := 8;
  PortDCB.Parity := NOPARITY;
  PortDCB.StopBits := ONESTOPBIT;
  PortDCB.Flags := FFlags;//4243;
  if (not SetCommState(FHandle, PortDCB)) then
    exit;
  Timeouts.ReadIntervalTimeout := MAXDWORD;
  Timeouts.ReadTotalTimeoutMultiplier := MAXDWORD;
  Timeouts.ReadTotalTimeoutConstant := FReadTimeOut;
  Timeouts.WriteTotalTimeoutMultiplier := MAXDWORD;
  Timeouts.WriteTotalTimeoutConstant := FWriteTimeOut;
  if (not SetCommTimeouts(FHandle, Timeouts)) then
    exit;
  EscapeCommFunction(FHandle, 11);
  Result := True;
end;

procedure TComPort.CloseSocket();
begin
  CloseHandle(FHandle);
  FHandle := INVALID_HANDLE_VALUE;
end;

function TComPort.Read() : String;
var
  dwRead : Cardinal;
begin
  Result := '';
  ReadFile(FHandle, m_pBuf^, 1024, dwRead, nil);
  Result := String(m_pBuf);
  SetLength(Result, dwRead);
end;

procedure  TComPort.SendByte(data: byte);
begin
   Write(char(data));
end;

function TComPort.RecvByte(ignored: integer) : byte;
var
  dwRead : Cardinal;
begin
  ReadFile(FHandle, m_pBuf^, 1, dwRead, nil);
  result := byte(m_pBuf^);
end;

Function TComPort.Write(AData : string) : boolean;
var
  m_ppBuf : Pointer;
  dwWrote, dwCurWrote : Cardinal;
begin
  Result := False;
  m_ppBuf := @Pointer(AData)^;
  dwWrote := 0;
  while (dwWrote < Length(AData)) do
  begin
    if (not WriteFile(FHandle, Pointer(LongInt(m_ppBuf) + dwWrote)^, length(AData) - dwWrote, dwCurWrote, nil)) then
    begin
      exit;
    end;
    Inc(dwWrote, dwCurWrote);
  end;
  Result := True;
end;

end.
